<?php
class WPJAM_Format_API{
	public static function get_parameters($format){
		if($format == 'gallery'){
			$text		= wpjam_get_parameter('text',	['method'=>'POST',	'sanitize_callback'=>'sanitize_textarea_field']);
			$images		= wpjam_get_parameter('images',	['method'=>'POST',	'required'=>true,	'sanitize_callback'=>[self::class, 'gallery_sanitize_callback']]);

			$content	= compact('images', 'text');
		}elseif($format == 'video'){
			$image		= wpjam_get_parameter('image',	['method'=>'POST',	'required'=>true,	'sanitize_callback'=>'esc_url_raw']);
			$video		= wpjam_get_parameter('video',	['method'=>'POST',	'required'=>true,	'sanitize_callback'=>'esc_url_raw']);
			$text		= wpjam_get_parameter('text',	['method'=>'POST',	'sanitize_callback'=>'sanitize_textarea_field']);
			$content	= compact('image', 'video', 'text');
		}elseif($format == 'image'){
			$content	= wpjam_get_parameter('images',	['method'=>'POST', 'required'=>true,	'sanitize_callback'=>[self::class, 'image_sanitize_callback']]);
		}else{
			return new WP_Error('invalid_format', '无效的文章格式');
		}

		$post_content	= maybe_serialize(wp_unslash($content));
		$post_title		= wpjam_get_parameter('title',	['method'=>'POST', 'required'=>true]);
		$post_status	= wpjam_get_parameter('status',	['method'=>'POST', 'required'=>true]);
		$category_ids	= wpjam_get_parameter('category_ids',	['method'=>'POST', 'required'=>true]);

		if(!current_user_can('publish_posts') && $post_status == 'publish'){
			$post_status = 'pending';
		}

		$tax_input		= ['category'=>$category_ids];

		return compact('post_title', 'post_content', 'post_status', 'tax_input');
	}

	public static function gallery_sanitize_callback($value){
		return array_filter(array_map('esc_url_raw', $value));
	}

	public static function image_sanitize_callback($value){
		return array_map(function($item){
			$item['image']	= esc_url_raw($item['image']);
			$item['text']	= sanitize_textarea_field($item['text']);

			return array_filter($item);
		}, $value);

		return array_filter($value);
	}

	public static function api_create_post(){
		if(!is_user_logged_in() || !current_user_can('edit_posts')){
			return new WP_Error('bad_authentication', '无权限');
		}

		$format	= wpjam_get_parameter('format', ['method'=>'POST', 'required'=>true]);
		$data	= self::get_parameters($format);

		if(is_wp_error($data)){
			return $post_content;
		}

		$post_id	= WPJAM_Post::insert(array_merge($data+['post_author'=>get_current_user_id()]));

		if(is_wp_error($post_id)){
			return $post_id;
		}

		set_post_format($post_id, $format);

		$post_status	= get_post_status($post_id);

		if($post_status == 'publish'){
			$errmsg	= '文章发布成功';
		}elseif($post_status == 'draft'){
			$errmsg	= '草稿创建成功';
		}elseif($post_status == 'pending'){
			$errmsg	= '文章等待待审中';
		}

		return [
			'errcode'		=> 0,
			'errmsg'		=> $errmsg,
			'post_id'		=> $post_id,
			'post_status'	=> $post_status
		];
	}

	public static function api_update_post(){
		if(!is_user_logged_in() || !current_user_can('edit_posts')){
			return new WP_Error('bad_authentication', '无权限');
		}

		$post_id	= wpjam_get_parameter('post_id', ['method'=>'POST', 'type'=>'int', 'required'=>true]);
		$the_post	= wpjam_validate_post($post_id);

		if(is_wp_error($the_post)){
			return $the_post;
		}

		$user_id	= get_current_user_id();

		if($user_id != $the_post->post_author){
			return new WP_Error('not_allowed', '无权限');
		}

		$old_status	= $the_post->post_status ;

		if(!current_user_can('edit_published_posts') && $old_status == 'publish'){
			return new WP_Error('not_allowed', '你无权修改已发布的文章');
		}

		$format	= get_post_format($the_post);
		$data	= self::get_parameters($format);

		if(is_wp_error($data)){
			return $data;
		}

		$post_id	= WPJAM_Post::update($post_id, $data);

		if(is_wp_error($post_id)){
			return $post_id;
		}

		$post_status	= get_post_status($post_id);

		if($post_status == 'publish'){
			$errmsg	= ($old_status == 'publish' ? '文章修改成功' : '文章发布成功');
		}elseif($post_status == 'draft'){
			$errmsg	= ($old_status == 'draft' ? '草稿修改成功' : '文章设置为草稿');
		}elseif($post_status == 'pending'){
			$errmsg	= '文章等待审中';
		}

		return [
			'errcode'		=> 0,
			'errmsg'		=> $errmsg,
			'post_id'		=> $post_id,
			'post_status'	=> $post_status
		];
	}

	public static function api_post_create_list(){
		if(!is_user_logged_in() || !current_user_can('edit_posts')){
			return new WP_Error('bad_authentication', '无权限');
		}

		return wpjam_parse_json_module([
			'type'	=> 'post_type',
			'args'	=> [
				'post_type'		=>'post',
				'post_status'	=>'any',
				'action'		=>'list',
				'thumbnail_size'=>'750x300',
				'author'		=>get_current_user_id(),
				'tax_query'		=>[['taxonomy'=>'post_format', 'field'=>'slug', 'terms'=>['post-format-image', 'post-format-gallery']]]
			]
		]);
	}

	public static function filter_post_json($post_json, $post_id){
		$post_json['editable']	= false;

		if(empty($post_json['format'])){
			return $post_json;
		}

		$format_content	= wpjam_get_format_content($post_id);

		if(empty($format_content)){
			$post_json['format']	= '';
			return $post_json;
		}

		if($user_id	= get_current_user_id()){
			if($post_json['user_id'] == $user_id){
				if($post_json['status'] == 'publish' && current_user_can('edit_published_posts')){
					$post_json['editable']	= true;
				}elseif(current_user_can('edit_posts')){
					$post_json['editable']	= true;
				}
			}
		}

		$format = $post_json['format'];

		if($format == 'image'){
			if(is_singular('post')){
				$images	= [];
				foreach($format_content as $item) {
					$image = $item['image'] ?? '';
					$images[]	=[
						'image'	=> $image ? wpjam_get_thumbnail($image, 1200) : '', 
						'raw'	=> $image,
						'text'	=> $item['text'] ?? ''
					];
				}

				$post_json['format_content']	= compact('images'); 
			}
		}elseif($format == 'gallery'){
			if(is_singular('post')){
				$images	= $format_content['images'] ?? [];
				$text	= $format_content['text'] ?? '';

				$gallery	= [];

				if($images){
					$count	= count($images);

					if($count == 1){
						$width	= 1200;
						$height	= 0;
					}elseif($count == 2 || $count == 4){
						$width	= $height = 400;
					}else{
						$width	= $height = 300;
					}

					$gallery	= array_map(function($image) use($width, $height){
						return [
							'thumbnail'	=> wpjam_get_thumbnail($image,  [$width, $height]), 
							'image'		=> wpjam_get_thumbnail($image, 1200),
							'raw'		=> wpjam_get_thumbnail($image)
						];
					}, $images);
				}

				$post_json['format_content']	= compact('gallery', 'text');
			}
		}elseif($format == 'link'){
			$text	= $format_content['text'] ?? '';
			$link	= $format_content['link'] ?? '';

			$post_json['format_content']	= compact('link', 'text');

			$post_json['editable']	= false;
		}elseif($format == 'video'){
			if(is_singular('post')){
				$text	= $format_content['text'] ?? '';
				$video	= $format_content['video'] ?? '';
				$raw	= $format_content['image'] ?? '';
				$poster	= $raw ? wpjam_get_thumbnail($raw, '640x400') : '';
				
				$video	= wpjam_get_video_mp4($video);

				$post_json['format_content']	= compact('video', 'poster', 'text');
			}

			$post_json['editable']	= false;
		}elseif($format == 'audio'){
			if(is_singular('post')){
				$title	= $post_json['title'];
				$text	= $format_content['text'] ?? '';
				$audio	= $format_content['audio'] ?? '';
				$raw	= $format_content['image'] ?? '';
				$poster	= $raw ? wpjam_get_thumbnail($raw, '100x100') : '';

				$post_json['format_content']	= compact('audio', 'poster', 'text', 'title');
			}

			$post_json['editable']	= false;
		}else{
			$post_json['editable']	= false;
		}

		return $post_json;
	}

	public static function filter_related_post_json($post_json, $post_id){
		$format	= get_post_format($post_id);

		$post_json['format']	= $format;

		if($format == 'link'){
			$format_content	= wpjam_get_format_content($post_id);

			$text	= $format_content['text'] ?? '';
			$link	= $format_content['link'] ?? '';

			$post_json['post_url']	= $link;
			$post_json['format_content']	= compact('link', 'text');
		}

		return $post_json;
	}
}

add_action('wpjam_api',	function ($json){
	add_filter('wpjam_post_json',			['WPJAM_Format_API', 'filter_post_json'], 1, 2);
	add_filter('wpjam_related_post_json',	['WPJAM_Format_API', 'filter_related_post_json'], 10, 2);

	if($json == 'post.create'){
		wpjam_register_api($json, [
			'title'		=> '创建文章',
			'auth'		=> true,
			'callback'	=> ['WPJAM_Format_API', 'api_create_post']
		]);
	}elseif($json == 'post.update'){
		wpjam_register_api($json, [
			'title'		=> '编辑文章',
			'auth'		=> true,
			'callback'	=> ['WPJAM_Format_API', 'api_update_post']
		]);
	}elseif($json == 'post.create.list'){
		return wpjam_register_api($json, [
			'title'			=> '我的文章',
			'page_title'	=> '我的文章',
			'share_title'	=> '我的文章',
			'auth'			=> true,
			'callback'		=> ['WPJAM_Format_API', 'api_post_create_list']
		]);
	}
});